#container {
    display: flex;
    flex-flow: row wrap;
    justify-content: space-around;
}

#container > #settings {
    flex-basis: 20%;
}

#container > #chartContainer {
    flex-basis: 50%;
}

.collapsible {
  background-color: #eee;
  color: #444;
  cursor: pointer;
  padding: 18px;
  width: 100%;
  border: none;
  text-align: left;
  outline: none;
  font-size: 15px;
}

.active, .collapsible:hover {
  background-color: #ccc;
}

.content {
  padding: 0 18px;
  display: none;
  overflow: hidden;
  background-color: #f1f1f1;
}